#!/bin/perl
################################################################################
# Created       : Martin Foster
# Modified      : 28-Sep-2003
################################################################################
#
# Weblog - Weblog functionality built into Ethereal Realms as a news outlet
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use eRealms::Database;						# Database handler
use eRealms::Param;						# Parameter control
use eRealms::Table;						# Table handler
use eRealms::Template;						# Template handler


################################################################################
# Data Members
################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $param;							# Parameter handle
my $script;							# Script name

my %board;							# Board hash


################################################################################
# Program Area
################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new eRealms::Database();

	# Connect and fetch
	$database->Connect();

	# Parameter handling
	$param = new eRealms::Param($database, $cgi);
	$param->GetParam();

	# Capture scriptname
        $script = $cgi->url(-relative=>1);


	# Round about way
	if ((length($script) < 2)
	 || ($script =~ /\.(shtml|html)$/i))
	{
		# Assume that this is not the real script
		# Get name and only end
		$script = $0;
		$script =~ s/.*\/(.*)$/$1/gs;
	}


	# Retreive hash
	$database->GetHashBoard(\%board);


	# Display handler
	# Abreviated
	if ($script =~ /^quicklog/i) 
	{
		# Brain dead spacing
		print "<BR>\n";

		# Quick view
		QuickView($database, $cgi, $param, \%board);
	}

	# Stripped
	elsif ($script =~ /^extlog/i)
	{
		# Brain dead spacing
		print "<BR>\n";

		# Full view
		PostView($database, $cgi, $param, \%board, $board{'TagLnkBoard'}, 100);
	}

	# Complete
	else
	{
		# Document header
		print $cgi->header();
		$database->DocumentGetHeader();

		# Full view
		PostView($database, $cgi, $param, \%board);

		# Document footer
		$database->DocumentGetFooter();	
	}

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Post Viewer 
#
# Dirt simple functionality that exists simply to display posts in the
# weblog.

sub PostView 
{
	#####################
	# Data members
	my $database = shift;				# Database handle
	my $cgi      = shift;			 	# CGI handle	
	my $param    = shift;				# Parameter list
	my $board    = shift;				# Board hash
	my $url      = shift;				# Self referencing address
	my $num      = shift;				# Number to display

	my $res;					# Query results
	my $statement;					# Query handle
	my $table;					# Table handler
	my $tmpl;					# Template handler

	my $dat;					# Date/time stamp
	my $dis;					# Display amount widget
	my $fil;					# Filtering by topic
	my $nam;					# Discovered name
	my $top;					# Currently selected topic
	my $wid;					# Specific message

	my $sub;					# Sub information
	my $tit;					# Formatted title

	my $inl;					# Basic inline
	my $nxt;					# Next set
	my $prv;					# Previous set
	my $min;					# First in line
	my $max;					# Last in line
	my $nav;					# Navigational link

	my @opt = qw(10 20 30 40 50);			# Optional amount to display

	my %fil;					# Filter inline hash
	my %bio;					# Bio inline hash


	#####################
	# Program area

	# Instance assignment
	$table = new eRealms::Table($cgi);
	$tmpl  = new eRealms::Template($board);

	# Next and previous adjustment
	($min, $max) = $database->DataGet("SELECT 
		  MIN(LogIDNumber), 
		  MAX(LogIDNumber) 
		FROM Weblog;");

	# Definition
	$fil = (defined($param->{'FIL'})) ? "AND LogTopic='" . lc($param->{FIL}) . "'" : "";
	$wid = (defined($param->{'WID'})) ? "AND LogIDNumber=$param->{WID}::INT" : "";

	# Cleanup
	$param->Cleanup('WID');

	# Parameter handling
	$param->{'NUM'} = ((defined($param->{'NUM'})) && ($param->{'NUM'} =~ /^\d+$/)) ? $param->{'NUM'} : 10;
	$param->{'TOP'} = ((defined($param->{'TOP'})) && ($param->{'TOP'} =~ /^\d+$/)) ? $param->{'TOP'} : $max;

	# Assignment
	$url = (defined($url)) ? $url : $cgi->url(-absolute=>1);
	$num = (defined($num)) ? $num : $param->{'NUM'};

	# Display prep
	# Initial title
	$tmpl->ShowMain($board->{'TagTitleWeblog'});

	# Table arrange
	$table->MakeTop("95%");

	# Loop and display
	# Find and list
	# Prepare and execute	
	$database->Pull(\$statement, "SELECT
		  LogIDNumber AS \"LogIDNumber\",
		  PuppetName  AS \"PuppetName\",
		  LogTitle    AS \"LogTitle\",
		  LogTopic    AS \"LogTopic\",
		  LogPost     AS \"LogPost\",
		  getDate(LogTimestamp) AS \"LogTimestamp\"
		FROM Weblog
		WHERE LogIDNumber <= ?::INT
		$fil
		$wid
		ORDER BY LogIDNumber DESC
		LIMIT $num",

	 $param->{'TOP'}
	);

	# Loop through results
	while ($res = $statement->fetchrow_hashref())
	{
		# Value handling
		# Assign
		$nam =  $res->{'PuppetName'};
		$top =  $res->{'LogTopic'};

		# Pull login name
		($dat) = $res->{'LogTimestamp'};

		# Next and previous values
		$nxt = $res->{'LogIDNumber'};
		$prv = (defined($prv)) ? $prv : $res->{'LogIDNumber'};

		# Define hashes
		$bio{$nam} = (defined($bio{$nam})) ? $bio{$nam} : $param->Crypt(BCHAR=>$nam, BPOOL=>'system');
		$fil{$top} = (defined($fil{$top})) ? $fil{$top} : $param->Crypt($param->Flat(), FIL=>$top);


		# Create display
		# Title
		$tit = $tmpl->PassSub($res->{'LogTitle'});

		# Subtitle
		$sub = "[<B><A HREF=\"$url$fil{$top}\">" . uc($top) . "</A></B>]" 
		 . " $board->{TagHeadFrom} <A HREF=\"$board->{TagLnkBio}$bio{$nam}\" TARGET=\"_new\">$nam</A> "
		 . $board->{TagHeadDate} . " $dat";


		# Display
		# Header
		$table->MakeSingle($tit);
		$table->MakeSingle($sub);
		$table->MakeBlank();

		# Post
		$table->MakeSingle($res->{'LogPost'});
		$table->MakeBlank();
	}

	# Finish query
	$statement->finish();


	# Directional handling
	# Previous and last
	$prv = ($prv != $max) ? ($prv + ($num)) : $max;
	$nxt = ($nxt != $min) ? ($nxt-1) : ($min + ($num-1));
	$prv = $param->Crypt($param->Flat(), 'TOP', $prv);
	$nxt = $param->Crypt($param->Flat(), 'TOP', $nxt);

	# First and last
	$min = $param->Crypt($param->Flat(), 'TOP', ($min + ($num-1)));
	$max = $param->Crypt($param->Flat(), 'TOP', $max);


	# Generate nav link
	# Cleaner code
	$nav = "<A HREF=\"$url\">$board->{TagReset}</A> - "
	     . "<A HREF=\"$url$max\">$board->{TagFirst}</A> " 
	     . "<A HREF=\"$url$prv\">$board->{TagPrev}</A> " 
	     . "<A HREF=\"$url$nxt\">$board->{TagNext}</A> "
	     . "<A HREF=\"$url$min\">$board->{TagLast}</A> - ";

	# Cleanup
	$param->Cleanup('NUM');

	# Generate number restrictors
	foreach $inl (@opt)
	{
		# Assign and modify
		$tit =  $board->{'TagVar'};
		$tit =~ s/MTITLE/$inl/gs;

		# Create parameter
		$inl =  $param->Crypt($param->Flat(), 'NUM', $inl);

		# Add to widget display
		$nav = $nav . "<A HREF=\"$url$inl\">$tit</A> ";
	}

	# Final assigment
	$tit =  $board->{'TagTitle'};
	$tit =~ s/MTITLE/&nbsp;/gs;

	# Navigation link
	$table->MakeSingle($tit);
	$table->MakeSingle("<CENTER>$nav</CENTER>");


	# Display completion
	# End table
	$table->MakeBottom();
}


#####################
# Quick Viewer 
#
# Used in conjunction with SSI as to be able to display a part of the log
# without including the rest of the interface.  Very useful for front
# pages.

sub QuickView 
{
	#####################
	# Data members
	my $database = shift;				# Database handle
	my $cgi      = shift;			 	# CGI handle	
	my $param    = shift;				# Parameter list
	my $board    = shift;				# Board hash

	my $res;					# Query results
	my $statement;					# Query handle
	my $table;					# Table handler

	my $dat;					# Date/time stamp
	my $dis;					# Display amount widget
	my $fil;					# Filtering by topic
	my $nam;					# Discovered name
	my $num;					# Number to display
	my $top;					# Currently selected topic

	my $inl;					# Inline information
	my $sub;					# Sub information

	my %fil;					# Filter inline hash
	my %bio;					# Bio inline hash


	#####################
	# Program area

	# Instance assignment
	$table = new eRealms::Table($cgi);


	# Find and list
	# Prepare and execute	
	$database->Pull(\$statement, "SELECT
		  LogIDNumber AS \"LogIDNumber\",
		  PuppetName  AS \"PuppetName\",
		  LogTitle    AS \"LogTitle\",
		  LogTopic    AS \"LogTopic\",
		  LogPost     AS \"LogPost\",
		  GetDateTime(LogTimestamp) AS \"LogTimestamp\"
		FROM Weblog
		ORDER BY LogIDNumber DESC
		LIMIT 3");

	# Loop and display
	while ($res = $statement->fetchrow_hashref())
	{
		# Value handling
		# Assign
		$nam =  $res->{'PuppetName'};
		$top =  $res->{'LogTopic'};

		# Pull login name
		($dat) = $res->{'LogTimestamp'};

		# Define hashes
		$bio{$nam} = (defined($bio{$nam})) ? $bio{$nam} : $param->Crypt(BCHAR=>$nam, BPOOL=>'system');
		$fil{$top} = (defined($fil{$top})) ? $fil{$top} : $param->Crypt($param->Flat(), FIL=>$top);

		# Inline parameter
		$inl = $param->Crypt($param->Flat(), WID=>$res->{'LogIDNumber'});


		# Create display
		# Subtitle
		$sub = "<B><A HREF=\"$board->{TagLnkBoard}$fil{$top}\">" . uc($top) . "</A></B><BR>" 
		 . "<A HREF=\"$board->{TagLnkBoard}$inl\" TARGET=\"_new\">$res->{LogTitle}</A><BR> "
		 . "$board->{TagHeadFrom} <A HREF=\"$board->{TagLnkBio}$bio{$nam}\" TARGET=\"_new\">$nam</A> "
		 . "(" . $dat . ")";

		# Display
		print "<P>$sub</P>\n";
	}

	# Finish query
	$statement->finish();
}
